@@ -3,22 +3,15 @@
 //  - comments and doc comments
 //  - pragma value is [^;]+
 //
+use itertools::{peek_nth, PeekNth};
 use phf::phf_map;
-use std::fmt;
-use std::iter::Peekable;
-use std::str::CharIndices;
+use std::{fmt, str::CharIndices};
 use unicode_xid::UnicodeXID;
 
-use crate::pt::{CodeLocation, Comment, Loc};
+use crate::pt::{CodeLocation, Comment, CommentType, Loc};
 
 pub type Spanned<Token, Loc, Error> = Result<(Loc, Token, Loc), Error>;
 
-#[derive(Copy, Clone, PartialEq, Debug)]
-pub enum CommentType {
-    Line,
-    Block,
-}
-
 #[derive(Copy, Clone, PartialEq, Debug)]
 pub enum Token<'input> {
     Identifier(&'input str),
@@ -163,9 +156,8 @@ pub enum Token<'input> {
     Hours,
     Days,
     Weeks,
+    Gwei,
     Wei,
-    Szabo,
-    Finney,
     Ether,
 
     This,
@@ -197,13 +189,13 @@ impl<'input> fmt::Display for Token<'input> {
             Token::StringLiteral(s) => write!(f, "\"{}\"", s),
             Token::HexLiteral(hex) => write!(f, "{}", hex),
             Token::AddressLiteral(address) => write!(f, "{}", address),
-            Token::Number(base, exp) if exp.is_empty() => write!(f, "{}", base),
-            Token::Number(base, exp) => write!(f, "{}e{}", base, exp),
-            Token::RationalNumber(significand, mantissa, exp) if exp.is_empty() => {
-                write!(f, "{}.{}", significand, mantissa)
+            Token::Number(integer, exp) if exp.is_empty() => write!(f, "{}", integer),
+            Token::Number(integer, exp) => write!(f, "{}e{}", integer, exp),
+            Token::RationalNumber(integer, fraction, exp) if exp.is_empty() => {
+                write!(f, "{}.{}", integer, fraction)
             }
-            Token::RationalNumber(significand, mantissa, exp) => {
-                write!(f, "{}.{}e{}", significand, mantissa, exp)
+            Token::RationalNumber(integer, fraction, exp) => {
+                write!(f, "{}.{}e{}", integer, fraction, exp)
             }
             Token::HexNumber(n) => write!(f, "{}", n),
             Token::Uint(w) => write!(f, "uint{}", w),
@@ -312,9 +304,8 @@ impl<'input> fmt::Display for Token<'input> {
             Token::Hours => write!(f, "hours"),
             Token::Days => write!(f, "days"),
             Token::Weeks => write!(f, "weeks"),
+            Token::Gwei => write!(f, "gwei"),
             Token::Wei => write!(f, "wei"),
-            Token::Szabo => write!(f, "szabo"),
-            Token::Finney => write!(f, "finney"),
             Token::Ether => write!(f, "ether"),
             Token::This => write!(f, "this"),
             Token::As => write!(f, "as"),
@@ -339,7 +330,7 @@ impl<'input> fmt::Display for Token<'input> {
 
 pub struct Lexer<'input> {
     input: &'input str,
-    chars: Peekable<CharIndices<'input>>,
+    chars: PeekNth<CharIndices<'input>>,
     comments: &'input mut Vec<Comment>,
     file_no: usize,
     last_tokens: [Option<Token<'input>>; 2],
@@ -354,8 +345,6 @@ pub enum LexicalError {
     InvalidCharacterInHexLiteral(Loc, char),
     UnrecognisedToken(Loc, String),
     MissingExponent(Loc),
-    DoublePoints(Loc),
-    UnrecognisedDecimal(Loc),
     ExpectedFrom(Loc, String),
 }
 
@@ -371,15 +360,11 @@ impl fmt::Display for LexicalError {
             }
             LexicalError::MissingNumber(..) => write!(f, "missing number"),
             LexicalError::InvalidCharacterInHexLiteral(_, ch) => {
-                write!(f, "invalid character ‘{}’ in hex literal string", ch)
+                write!(f, "invalid character '{}' in hex literal string", ch)
             }
-            LexicalError::UnrecognisedToken(_, t) => write!(f, "unrecognised token ‘{}’", t),
-            LexicalError::ExpectedFrom(_, t) => write!(f, "‘{}’ found where ‘from’ expected", t),
+            LexicalError::UnrecognisedToken(_, t) => write!(f, "unrecognised token '{}'", t),
+            LexicalError::ExpectedFrom(_, t) => write!(f, "'{}' found where 'from' expected", t),
             LexicalError::MissingExponent(..) => write!(f, "missing number"),
-            LexicalError::DoublePoints(..) => write!(f, "found two dots in number"),
-            LexicalError::UnrecognisedDecimal(..) => {
-                write!(f, "expected number after decimal point")
-            }
         }
     }
 }
@@ -394,9 +379,7 @@ impl CodeLocation for LexicalError {
             | LexicalError::InvalidCharacterInHexLiteral(loc, _)
             | LexicalError::UnrecognisedToken(loc, ..)
             | LexicalError::ExpectedFrom(loc, ..)
-            | LexicalError::MissingExponent(loc, ..)
-            | LexicalError::DoublePoints(loc, ..)
-            | LexicalError::UnrecognisedDecimal(loc, ..) => *loc,
+            | LexicalError::MissingExponent(loc, ..) => *loc,
         }
     }
 }
@@ -561,8 +544,7 @@ static KEYWORDS: phf::Map<&'static str, Token> = phf_map! {
     "days" => Token::Days,
     "weeks" => Token::Weeks,
     "wei" => Token::Wei,
-    "szabo" => Token::Szabo,
-    "finney" => Token::Finney,
+    "gwei" => Token::Gwei,
     "ether" => Token::Ether,
     "this" => Token::This,
     "as" => Token::As,
@@ -582,7 +564,7 @@ impl<'input> Lexer<'input> {
     pub fn new(input: &'input str, file_no: usize, comments: &'input mut Vec<Comment>) -> Self {
         Lexer {
             input,
-            chars: input.char_indices().peekable(),
+            chars: peek_nth(input.char_indices()),
             comments,
             file_no,
             last_tokens: [None, None],
@@ -653,41 +635,22 @@ impl<'input> Lexer<'input> {
             rational_start = start + 1;
         }
 
-        if let Some((i, '.')) = self.chars.peek() {
-            if is_rational {
-                return Err(LexicalError::DoublePoints(Loc::File(
-                    self.file_no,
-                    start,
-                    self.input.len(),
-                )));
-            }
-            rational_start = *i + 1;
-            rational_end = *i + 1;
-            let mut has_number = false;
-            is_rational = true;
-            self.chars.next();
-            while let Some((i, ch)) = self.chars.peek() {
-                if *ch == '.' {
-                    return Err(LexicalError::DoublePoints(Loc::File(
-                        self.file_no,
-                        start,
-                        self.input.len(),
-                    )));
+        if let Some((_, '.')) = self.chars.peek() {
+            if let Some((i, ch)) = self.chars.peek_nth(1) {
+                if ch.is_ascii_digit() && !is_rational {
+                    rational_start = *i;
+                    rational_end = *i;
+                    is_rational = true;
+                    self.chars.next(); // advance over '.'
+                    while let Some((i, ch)) = self.chars.peek() {
+                        if !ch.is_ascii_digit() {
+                            break;
+                        }
+                        rational_end = *i;
+                        end = *i;
+                        self.chars.next();
+                    }
                 }
-                if !ch.is_ascii_digit() {
-                    break;
-                }
-                has_number = true;
-                rational_end = *i;
-                end = *i;
-                self.chars.next();
-            }
-            if !has_number {
-                return Err(LexicalError::UnrecognisedDecimal(Loc::File(
-                    self.file_no,
-                    start,
-                    self.input.len(),
-                )));
             }
         }
 
@@ -715,28 +678,21 @@ impl<'input> Lexer<'input> {
         }
 
         if is_rational {
-            let significand = &self.input[start..=end_before_rational];
-            let mantissa = &self.input[rational_start..=rational_end];
-
-            if mantissa.is_empty() {
-                return Err(LexicalError::UnrecognisedDecimal(Loc::File(
-                    self.file_no,
-                    start,
-                    self.input.len(),
-                )));
-            }
+            let integer = &self.input[start..=end_before_rational];
+            let fraction = &self.input[rational_start..=rational_end];
             let exp = &self.input[exp_start..=end];
+
             return Ok((
                 start,
-                Token::RationalNumber(significand, mantissa, exp),
+                Token::RationalNumber(integer, fraction, exp),
                 end + 1,
             ));
         }
 
-        let base = &self.input[start..=old_end];
+        let integer = &self.input[start..=old_end];
         let exp = &self.input[exp_start..=end];
 
-        Ok((start, Token::Number(base, exp), end + 1))
+        Ok((start, Token::Number(integer, exp), end + 1))
     }
 
     fn string(
@@ -954,11 +910,8 @@ impl<'input> Lexer<'input> {
                             // multiline comment
                             self.chars.next();
 
-                            let doc_comment_start = match self.chars.next() {
-                                Some((i, '*')) => match self.chars.peek() {
-                                    Some((_, '*')) => None,
-                                    _ => Some(i + 1),
-                                },
+                            let doc_comment_start = match self.chars.peek() {
+                                Some((i, '*')) => Some(i + 1),
                                 _ => None,
                             };
 
@@ -983,21 +936,25 @@ impl<'input> Lexer<'input> {
 
                             if let Some(doc_start) = doc_comment_start {
                                 if last > doc_start {
-                                    return Some(Ok((
-                                        start + 3,
-                                        Token::DocComment(
-                                            CommentType::Block,
-                                            &self.input[doc_start..last],
-                                        ),
-                                        last,
-                                    )));
+                                    let comment = &self.input[doc_start..last];
+
+                                    if comment.chars().any(|ch| !ch.is_whitespace() && ch != '*') {
+                                        return Some(Ok((
+                                            start + 3,
+                                            Token::DocComment(
+                                                CommentType::Block,
+                                                &self.input[doc_start..last],
+                                            ),
+                                            last,
+                                        )));
+                                    }
                                 }
-                            } else {
-                                self.comments.push(Comment::Block(
-                                    Loc::File(self.file_no, start, last + 2),
-                                    self.input[start..last + 2].to_owned(),
-                                ));
                             }
+
+                            self.comments.push(Comment::Block(
+                                Loc::File(self.file_no, start, last + 2),
+                                self.input[start..last + 2].to_owned(),
+                            ));
                         }
                         _ => {
                             return Some(Ok((start, Token::Divide, start + 1)));
@@ -1612,4 +1569,29 @@ fn lexertest() {
             Ok((2, Token::Identifier("a"), 3))
         )
     );
+
+    let tokens = Lexer::new(r#"42.a"#, 0, &mut comments)
+        .collect::<Vec<Result<(usize, Token, usize), LexicalError>>>();
+
+    assert_eq!(
+        tokens,
+        vec!(
+            Ok((0, Token::Number("42", ""), 2)),
+            Ok((2, Token::Member, 3)),
+            Ok((3, Token::Identifier("a"), 4))
+        )
+    );
+
+    let tokens = Lexer::new(r#"42..a"#, 0, &mut comments)
+        .collect::<Vec<Result<(usize, Token, usize), LexicalError>>>();
+
+    assert_eq!(
+        tokens,
+        vec!(
+            Ok((0, Token::Number("42", ""), 2)),
+            Ok((2, Token::Member, 3)),
+            Ok((3, Token::Member, 4)),
+            Ok((4, Token::Identifier("a"), 5))
+        )
+    );
 }
