@@ -6,7 +6,6 @@ use lalrpop_util::ParseError;
 pub use diagnostics::Diagnostic;
 
 pub mod diagnostics;
-mod doc;
 pub mod lexer;
 pub mod pt;
 #[cfg(test)]
@@ -41,7 +40,7 @@ pub fn parse(
             } => Diagnostic::parser_error(
                 pt::Loc::File(file_no, l, r),
                 format!(
-                    "unrecognised token `{}', expected {}",
+                    "unrecognised token '{}', expected {}",
                     token,
                     expected.join(", ")
                 ),
@@ -49,7 +48,7 @@ pub fn parse(
             ParseError::User { error } => Diagnostic::parser_error(error.loc(), error.to_string()),
             ParseError::ExtraToken { token } => Diagnostic::parser_error(
                 pt::Loc::File(file_no, token.0, token.2),
-                format!("extra token `{}' encountered", token.0),
+                format!("extra token '{}' encountered", token.0),
             ),
             ParseError::UnrecognizedEOF { location, expected } => Diagnostic::parser_error(
                 pt::Loc::File(file_no, location, location),
