@@ -1,9 +1,4 @@
-use std::fmt;
-
-use num_bigint::BigInt;
-use num_rational::BigRational;
-
-use crate::lexer::CommentType;
+use std::fmt::{self, Display};
 
 #[derive(Debug, PartialEq, PartialOrd, Ord, Eq, Hash, Clone, Copy)]
 /// file no, start offset, end offset (in bytes)
@@ -94,26 +89,17 @@ pub enum Comment {
     Block(Loc, String),
 }
 
-#[derive(Debug, PartialEq, Clone)]
-pub enum DocComment {
-    Line { comment: SingleDocComment },
-    Block { comments: Vec<SingleDocComment> },
-}
-
-impl DocComment {
-    pub fn comments(&self) -> Vec<&SingleDocComment> {
-        match self {
-            DocComment::Line { comment } => vec![comment],
-            DocComment::Block { comments } => comments.iter().collect(),
-        }
-    }
+#[derive(Copy, Clone, PartialEq, Debug)]
+pub enum CommentType {
+    Line,
+    Block,
 }
 
 #[derive(Debug, PartialEq, Clone)]
-pub struct SingleDocComment {
-    pub offset: usize,
-    pub tag: String,
-    pub value: String,
+pub struct DocComment {
+    pub loc: Loc,
+    pub ty: CommentType,
+    pub comment: String,
 }
 
 #[derive(Debug, PartialEq, Clone)]
@@ -122,8 +108,8 @@ pub struct SourceUnit(pub Vec<SourceUnitPart>);
 #[derive(Debug, PartialEq, Clone)]
 pub enum SourceUnitPart {
     ContractDefinition(Box<ContractDefinition>),
-    PragmaDirective(Loc, Vec<DocComment>, Identifier, StringLiteral),
-    ImportDirective(Vec<DocComment>, Import),
+    PragmaDirective(Loc, Identifier, StringLiteral),
+    ImportDirective(Import),
     EnumDefinition(Box<EnumDefinition>),
     StructDefinition(Box<StructDefinition>),
     EventDefinition(Box<EventDefinition>),
@@ -131,6 +117,8 @@ pub enum SourceUnitPart {
     FunctionDefinition(Box<FunctionDefinition>),
     VariableDefinition(Box<VariableDefinition>),
     TypeDefinition(Box<TypeDefinition>),
+    DocComment(DocComment),
+    Using(Box<Using>),
     StraySemicolon(Loc),
 }
 
@@ -141,6 +129,8 @@ pub enum Import {
     Rename(StringLiteral, Vec<(Identifier, Option<Identifier>)>, Loc),
 }
 
+pub type ParameterList = Vec<(Loc, Option<Parameter>)>;
+
 #[derive(Debug, PartialEq, Clone)]
 pub enum Type {
     Address,
@@ -157,8 +147,7 @@ pub enum Type {
     Function {
         params: Vec<(Loc, Option<Parameter>)>,
         attributes: Vec<FunctionAttribute>,
-        returns: Vec<(Loc, Option<Parameter>)>,
-        trailing_attributes: Vec<FunctionAttribute>,
+        returns: Option<(ParameterList, Vec<FunctionAttribute>)>,
     },
 }
 
@@ -200,7 +189,6 @@ pub struct VariableDeclaration {
 #[derive(Debug, PartialEq, Clone)]
 #[allow(clippy::vec_box)]
 pub struct StructDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub name: Identifier,
     pub fields: Vec<VariableDeclaration>,
@@ -217,13 +205,21 @@ pub enum ContractPart {
     TypeDefinition(Box<TypeDefinition>),
     StraySemicolon(Loc),
     Using(Box<Using>),
+    DocComment(DocComment),
+}
+
+#[derive(Debug, PartialEq, Clone)]
+pub enum UsingList {
+    Library(Expression),
+    Functions(Vec<Expression>),
 }
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct Using {
     pub loc: Loc,
-    pub library: Identifier,
+    pub list: UsingList,
     pub ty: Option<Expression>,
+    pub global: Option<Identifier>,
 }
 
 #[derive(Debug, PartialEq, Clone)]
@@ -248,13 +244,12 @@ impl fmt::Display for ContractTy {
 #[derive(Debug, PartialEq, Clone)]
 pub struct Base {
     pub loc: Loc,
-    pub name: Identifier,
+    pub name: Expression,
     pub args: Option<Vec<Expression>>,
 }
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct ContractDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub ty: ContractTy,
     pub name: Identifier,
@@ -272,7 +267,6 @@ pub struct EventParameter {
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct EventDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub name: Identifier,
     pub fields: Vec<EventParameter>,
@@ -288,7 +282,6 @@ pub struct ErrorParameter {
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct ErrorDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub name: Identifier,
     pub fields: Vec<ErrorParameter>,
@@ -296,7 +289,6 @@ pub struct ErrorDefinition {
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct EnumDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub name: Identifier,
     pub values: Vec<Identifier>,
@@ -312,7 +304,6 @@ pub enum VariableAttribute {
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct VariableDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub ty: Expression,
     pub attrs: Vec<VariableAttribute>,
@@ -322,7 +313,6 @@ pub struct VariableDefinition {
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct TypeDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub name: Identifier,
     pub ty: Expression,
@@ -355,8 +345,7 @@ pub enum Unit {
     Days(Loc),
     Weeks(Loc),
     Wei(Loc),
-    Szabo(Loc),
-    Finney(Loc),
+    Gwei(Loc),
     Ether(Loc),
 }
 
@@ -415,15 +404,15 @@ pub enum Expression {
     AssignDivide(Loc, Box<Expression>, Box<Expression>),
     AssignModulo(Loc, Box<Expression>, Box<Expression>),
     BoolLiteral(Loc, bool),
-    NumberLiteral(Loc, BigInt),
-    RationalNumberLiteral(Loc, BigRational),
+    NumberLiteral(Loc, String, String),
+    RationalNumberLiteral(Loc, String, String, String),
     HexNumberLiteral(Loc, String),
     StringLiteral(Vec<StringLiteral>),
     Type(Loc, Type),
     HexLiteral(Vec<HexLiteral>),
     AddressLiteral(Loc, String),
     Variable(Identifier),
-    List(Loc, Vec<(Loc, Option<Parameter>)>),
+    List(Loc, ParameterList),
     ArrayLiteral(Loc, Vec<Expression>),
     Unit(Loc, Box<Expression>, Unit),
     This(Loc),
@@ -480,8 +469,8 @@ impl CodeLocation for Expression {
             | Expression::AssignDivide(loc, ..)
             | Expression::AssignModulo(loc, ..)
             | Expression::BoolLiteral(loc, _)
-            | Expression::NumberLiteral(loc, _)
-            | Expression::RationalNumberLiteral(loc, _)
+            | Expression::NumberLiteral(loc, ..)
+            | Expression::RationalNumberLiteral(loc, ..)
             | Expression::HexNumberLiteral(loc, _)
             | Expression::ArrayLiteral(loc, _)
             | Expression::List(loc, _)
@@ -496,6 +485,16 @@ impl CodeLocation for Expression {
     }
 }
 
+impl Display for Expression {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Expression::Variable(id) => write!(f, "{}", id.name),
+            Expression::MemberAccess(_, e, id) => write!(f, "{}.{}", e, id.name),
+            _ => unimplemented!(),
+        }
+    }
+}
+
 #[derive(Clone, Debug, PartialEq)]
 pub struct Parameter {
     pub loc: Loc,
@@ -568,6 +567,7 @@ pub enum FunctionAttribute {
     Mutability(Mutability),
     Visibility(Visibility),
     Virtual(Loc),
+    Immutable(Loc),
     Override(Loc, Vec<Identifier>),
     BaseOrModifier(Loc, Base),
 }
@@ -595,15 +595,14 @@ impl fmt::Display for FunctionTy {
 
 #[derive(Debug, PartialEq, Clone)]
 pub struct FunctionDefinition {
-    pub doc: Vec<DocComment>,
     pub loc: Loc,
     pub ty: FunctionTy,
     pub name: Option<Identifier>,
     pub name_loc: Loc,
-    pub params: Vec<(Loc, Option<Parameter>)>,
+    pub params: ParameterList,
     pub attributes: Vec<FunctionAttribute>,
     pub return_not_returns: Option<Loc>,
-    pub returns: Vec<(Loc, Option<Parameter>)>,
+    pub returns: ParameterList,
     pub body: Option<Statement>,
 }
 
@@ -636,15 +635,16 @@ pub enum Statement {
     Continue(Loc),
     Break(Loc),
     Return(Loc, Option<Expression>),
-    Revert(Loc, Option<Identifier>, Vec<Expression>),
+    Revert(Loc, Option<Expression>, Vec<Expression>),
+    RevertNamedArgs(Loc, Option<Expression>, Vec<NamedArgument>),
     Emit(Loc, Expression),
     Try(
         Loc,
         Expression,
-        Option<(Vec<(Loc, Option<Parameter>)>, Box<Statement>)>,
+        Option<(ParameterList, Box<Statement>)>,
         Vec<CatchClause>,
     ),
-    DocComment(Loc, CommentType, String),
+    DocComment(DocComment),
 }
 
 #[derive(Debug, PartialEq, Clone)]
@@ -693,7 +693,7 @@ pub struct YulBlock {
 #[derive(Debug, PartialEq, Clone)]
 pub enum YulExpression {
     BoolLiteral(Loc, bool, Option<Identifier>),
-    NumberLiteral(Loc, BigInt, Option<Identifier>),
+    NumberLiteral(Loc, String, String, Option<Identifier>),
     HexNumberLiteral(Loc, String, Option<Identifier>),
     HexStringLiteral(HexLiteral, Option<Identifier>),
     StringLiteral(StringLiteral, Option<Identifier>),
@@ -755,9 +755,10 @@ impl CodeLocation for Statement {
             | Statement::Break(loc)
             | Statement::Return(loc, ..)
             | Statement::Revert(loc, ..)
+            | Statement::RevertNamedArgs(loc, ..)
             | Statement::Emit(loc, ..)
-            | Statement::Try(loc, ..)
-            | Statement::DocComment(loc, ..) => *loc,
+            | Statement::Try(loc, ..) => *loc,
+            Statement::DocComment(comment) => comment.loc,
         }
     }
 }
