@@ -1,7 +1,6 @@
-use crate::lexer;
+use crate::lexer::Lexer;
 use crate::pt::*;
 use crate::solidity;
-use num_bigint::BigInt;
 
 #[test]
 fn parse_test() {
@@ -41,30 +40,17 @@ fn parse_test() {
                 }"#;
 
     let mut comments = Vec::new();
-    let lex = lexer::Lexer::new(src, 0, &mut comments);
+    let lex = Lexer::new(src, 0, &mut comments);
 
     let actual_parse_tree = solidity::SourceUnitParser::new()
         .parse(src, 0, lex)
         .unwrap();
 
     let expected_parse_tree = SourceUnit(vec![
+        SourceUnitPart::DocComment(DocComment{ loc: Loc::File(0,3, 14), ty: CommentType::Line, comment: " @title Foo".to_string()}),
+        SourceUnitPart::DocComment(DocComment{ loc: Loc::File(0,34,51), ty: CommentType::Line, comment: " @description Foo".to_string()}),
+        SourceUnitPart::DocComment(DocComment{ loc: Loc::File(0,71,75), ty: CommentType::Line, comment: " Bar".to_string()}),
         SourceUnitPart::ContractDefinition(Box::new(ContractDefinition {
-            doc: vec![
-                DocComment::Line {
-                    comment: SingleDocComment {
-                        offset: 0,
-                        tag: "title".to_string(),
-                        value: "Foo".to_string(),
-                    },
-                },
-                DocComment::Line {
-                    comment: SingleDocComment {
-                        offset: 0,
-                        tag: "description".to_string(),
-                        value: "Foo\nBar".to_string(),
-                    },
-                },
-            ],
             loc: Loc::File(0, 92, 105),
             ty: ContractTy::Contract(Loc::File(0, 92, 100)),
             name: Identifier {
@@ -73,37 +59,10 @@ fn parse_test() {
             },
             base: Vec::new(),
             parts: vec![
+                ContractPart::DocComment(DocComment{ loc: Loc::File(0,130,191), ty: CommentType::Block, comment: "\n                    @title Jurisdiction\n                    ".to_string()}),
+                ContractPart::DocComment(DocComment{ loc: Loc::File(0,217,230), ty: CommentType::Line, comment: " @author Anon".to_string()}),
+                ContractPart::DocComment(DocComment{ loc: Loc::File(0,254,389), ty: CommentType::Block, comment: "\n                    @description Data for\n                    jurisdiction\n                    @dev It's a struct\n                    ".to_string()}),
                 ContractPart::StructDefinition(Box::new(StructDefinition {
-                    doc: vec![
-                        DocComment::Block {
-                            comments: vec![SingleDocComment {
-                                offset: 0,
-                                tag: "title".to_string(),
-                                value: "Jurisdiction".to_string(),
-                            }],
-                        },
-                        DocComment::Line {
-                            comment: SingleDocComment {
-                                offset: 0,
-                                tag: "author".to_string(),
-                                value: "Anon".to_string(),
-                            },
-                        },
-                        DocComment::Block {
-                            comments: vec![
-                                SingleDocComment {
-                                    offset: 0,
-                                    tag: "description".to_string(),
-                                    value: "Data for\njurisdiction".to_string(),
-                                },
-                                SingleDocComment {
-                                    offset: 0,
-                                    tag: "dev".to_string(),
-                                    value: "It's a struct".to_string(),
-                                },
-                            ],
-                        },
-                    ],
                     name: Identifier {
                         loc: Loc::File(0, 419, 431),
                         name: "Jurisdiction".to_string(),
@@ -149,7 +108,6 @@ fn parse_test() {
                     ],
                 })),
                 ContractPart::VariableDefinition(Box::new(VariableDefinition {
-                    doc: vec![],
                     ty: Expression::Type(Loc::File(0, 630, 636), Type::String),
                     attrs: vec![],
                     name: Identifier {
@@ -160,7 +118,6 @@ fn parse_test() {
                     initializer: None,
                 })),
                 ContractPart::VariableDefinition(Box::new(VariableDefinition {
-                    doc: vec![],
                     ty: Expression::Type(Loc::File(0, 667, 672), Type::Int(64)),
                     attrs: vec![],
                     name: Identifier {
@@ -173,7 +130,6 @@ fn parse_test() {
             ],
         })),
         SourceUnitPart::FunctionDefinition(Box::new(FunctionDefinition {
-            doc: vec![],
             loc: Loc::File(0, 720, 735),
             ty: FunctionTy::Function,
             name: Some(Identifier {
@@ -197,8 +153,8 @@ fn parse_test() {
                             name: "sum".to_string(),
                         })),
                         vec![
-                            Expression::NumberLiteral(Loc::File(0, 765, 766), 1.into()),
-                            Expression::NumberLiteral(Loc::File(0, 768, 769), 1.into()),
+                            Expression::NumberLiteral(Loc::File(0, 765, 766), "1".to_string(), "".to_string()),
+                            Expression::NumberLiteral(Loc::File(0, 768, 769), "1".to_string(), "".to_string()),
                         ],
                     ),
                     Some((
@@ -233,7 +189,7 @@ fn parse_test() {
                                         })),
                                         Box::new(Expression::NumberLiteral(
                                             Loc::File(0, 830, 831),
-                                            2.into(),
+                                            "2".to_string(), "".to_string(),
                                         )),
                                     )],
                                 ),
@@ -349,224 +305,98 @@ fn parse_error_test() {
     let (actual_parse_tree, _) = crate::parse(src, 0).unwrap();
     assert_eq!(actual_parse_tree.0.len(), 2);
 
-    let expected_parse_tree = SourceUnit
-            (vec![
-                SourceUnitPart::ErrorDefinition(Box::new(ErrorDefinition {
-                    doc: vec![],
-                    loc: Loc::File(
-                        0,
-                        10,
-                        58,
-                    ),
+    let expected_parse_tree = SourceUnit(vec![
+        SourceUnitPart::ErrorDefinition(Box::new(ErrorDefinition {
+            loc: Loc::File(0, 10, 58),
+            name: Identifier {
+                loc: Loc::File(0, 16, 21),
+                name: "Outer".to_string(),
+            },
+            fields: vec![
+                ErrorParameter {
+                    ty: Expression::Type(Loc::File(0, 22, 29), Type::Uint(256)),
+                    loc: Loc::File(0, 22, 39),
+                    name: Some(Identifier {
+                        loc: Loc::File(0, 30, 39),
+                        name: "available".to_string(),
+                    }),
+                },
+                ErrorParameter {
+                    ty: Expression::Type(Loc::File(0, 41, 48), Type::Uint(256)),
+                    loc: Loc::File(0, 41, 57),
+                    name: Some(Identifier {
+                        loc: Loc::File(0, 49, 57),
+                        name: "required".to_string(),
+                    }),
+                },
+            ],
+        })),
+        SourceUnitPart::ContractDefinition(Box::new(ContractDefinition {
+            loc: Loc::File(0, 69, 88),
+            ty: ContractTy::Contract(Loc::File(0, 69, 77)),
+            name: Identifier {
+                loc: Loc::File(0, 78, 87),
+                name: "TestToken".to_string(),
+            },
+            base: vec![],
+            parts: vec![
+                ContractPart::ErrorDefinition(Box::new(ErrorDefinition {
+                    loc: Loc::File(0, 102, 120),
                     name: Identifier {
-                        loc: Loc::File(
-                            0,
-                            16,
-                            21,
-                        ),
-                        name: "Outer".to_string(),
+                        loc: Loc::File(0, 108, 118),
+                        name: "NotPending".to_string(),
+                    },
+                    fields: vec![],
+                })),
+                ContractPart::DocComment(DocComment {
+                    loc: Loc::File(0, 137, 199),
+                    ty: CommentType::Line,
+                    comment: " Insufficient balance for transfer. Needed `required` but only"
+                        .to_string(),
+                }),
+                ContractPart::DocComment(DocComment {
+                    loc: Loc::File(0, 215, 238),
+                    ty: CommentType::Line,
+                    comment: " `available` available.".to_string(),
+                }),
+                ContractPart::DocComment(DocComment {
+                    loc: Loc::File(0, 254, 290),
+                    ty: CommentType::Line,
+                    comment: " @param available balance available.".to_string(),
+                }),
+                ContractPart::DocComment(DocComment {
+                    loc: Loc::File(0, 306, 352),
+                    ty: CommentType::Line,
+                    comment: " @param required requested amount to transfer.".to_string(),
+                }),
+                ContractPart::ErrorDefinition(Box::new(ErrorDefinition {
+                    loc: Loc::File(0, 365, 427),
+                    name: Identifier {
+                        loc: Loc::File(0, 371, 390),
+                        name: "InsufficientBalance".to_string(),
                     },
                     fields: vec![
                         ErrorParameter {
-                            ty: Expression::Type(
-                                Loc::File(
-                                    0,
-                                    22,
-                                    29,
-                                ),
-                                Type::Uint(
-                                    256,
-                                ),
-                            ),
-                            loc: Loc::File(
-                                0,
-                                22,
-                                39,
-                            ),
-                            name: Some(
-                                Identifier {
-                                    loc: Loc::File(
-                                        0,
-                                        30,
-                                        39,
-                                    ),
-                                    name: "available".to_string(),
-                                },
-                            ),
+                            ty: Expression::Type(Loc::File(0, 391, 398), Type::Uint(256)),
+                            loc: Loc::File(0, 391, 408),
+                            name: Some(Identifier {
+                                loc: Loc::File(0, 399, 408),
+                                name: "available".to_string(),
+                            }),
                         },
                         ErrorParameter {
-                            ty: Expression::Type(
-                                Loc::File(
-                                    0,
-                                    41,
-                                    48,
-                                ),
-                                Type::Uint(
-                                    256,
-                                ),
-                            ),
-                            loc: Loc::File(
-                                0,
-                                41,
-                                57,
-                            ),
-                            name: Some(
-                                Identifier {
-                                    loc: Loc::File(
-                                        0,
-                                        49,
-                                        57,
-                                    ),
-                                    name: "required".to_string(),
-                                },
-                            ),
+                            ty: Expression::Type(Loc::File(0, 410, 417), Type::Uint(256)),
+                            loc: Loc::File(0, 410, 426),
+                            name: Some(Identifier {
+                                loc: Loc::File(0, 418, 426),
+                                name: "required".to_string(),
+                            }),
                         },
                     ],
                 })),
-                SourceUnitPart::ContractDefinition(Box::new(
-                    ContractDefinition {
-                        doc: vec![],
-                        loc: Loc::File(
-                            0,
-                            69,
-                            88,
-                        ),
-                        ty: ContractTy::Contract(
-                            Loc::File(
-                                0,
-                                69,
-                                77,
-                            ),
-                        ),
-                        name: Identifier {
-                            loc: Loc::File(
-                                0,
-                                78,
-                                87,
-                            ),
-                            name: "TestToken".to_string(),
-                        },
-                        base: vec![],
-                        parts: vec![
-                            ContractPart::ErrorDefinition(Box::new(
-                                ErrorDefinition {
-                                    doc: vec![],
-                                    loc: Loc::File(
-                                        0,
-                                        102,
-                                        120,
-                                    ),
-                                    name: Identifier {
-                                        loc: Loc::File(
-                                            0,
-                                            108,
-                                            118,
-                                        ),
-                                        name: "NotPending".to_string(),
-                                    },
-                                    fields: vec![],
-                                },
-                            )),
-                            ContractPart::ErrorDefinition(Box::new(
-                                ErrorDefinition {
-                                    doc: vec![
-                                        DocComment::Line {
-                                            comment: SingleDocComment {
-                                                offset: 137,
-                                                tag: "notice".to_string(),
-                                                value: "Insufficient balance for transfer. Needed `required` but only\n`available` available.".to_string(),
-                                            },
-                                        },
-                                        DocComment::Line {
-                                            comment: SingleDocComment {
-                                                offset: 0,
-                                                tag: "param".to_string(),
-                                                value: "available balance available.".to_string(),
-                                            },
-                                        },
-                                        DocComment::Line {
-                                            comment: SingleDocComment {
-                                                offset: 0,
-                                                tag: "param".to_string(),
-                                                value: "required requested amount to transfer.".to_string(),
-                                            },
-                                        },
-                                    ],
-                                    loc: Loc::File(
-                                        0,
-                                        365,
-                                        427,
-                                    ),
-                                    name: Identifier {
-                                        loc: Loc::File(
-                                            0,
-                                            371,
-                                            390,
-                                        ),
-                                        name: "InsufficientBalance".to_string(),
-                                    },
-                                    fields: vec![
-                                        ErrorParameter {
-                                            ty: Expression::Type(
-                                                Loc::File(
-                                                    0,
-                                                    391,
-                                                    398,
-                                                ),
-                                                Type::Uint(
-                                                    256,
-                                                ),
-                                            ),
-                                            loc: Loc::File(
-                                                0,
-                                                391,
-                                                408,
-                                            ),
-                                            name: Some(
-                                                Identifier {
-                                                    loc: Loc::File(
-                                                        0,
-                                                        399,
-                                                        408,
-                                                    ),
-                                                    name: "available".to_string(),
-                                                },
-                                            ),
-                                        },
-                                        ErrorParameter {
-                                            ty: Expression::Type(
-                                                Loc::File(
-                                                    0,
-                                                    410,
-                                                    417,
-                                                ),
-                                                Type::Uint(
-                                                    256,
-                                                ),
-                                            ),
-                                            loc: Loc::File(
-                                                0,
-                                                410,
-                                                426,
-                                            ),
-                                            name: Some(
-                                                Identifier {
-                                                    loc: Loc::File(
-                                                        0,
-                                                        418,
-                                                        426,
-                                                    ),
-                                                    name: "required".to_string(),
-                                                },
-                                            ),
-                                        },
-                                    ],
-                                },
-                            )),
-                        ],
-                    },
-                ))
-            ]);
+            ],
+        })),
+    ]);
 
     assert_eq!(actual_parse_tree, expected_parse_tree);
 }
@@ -608,14 +438,13 @@ fn test_assembly_parser() {
                 }"#;
 
     let mut comments = Vec::new();
-    let lex = lexer::Lexer::new(src, 0, &mut comments);
+    let lex = Lexer::new(src, 0, &mut comments);
     let actual_parse_tree = solidity::SourceUnitParser::new()
         .parse(src, 0, lex)
         .unwrap();
 
     let expected_parse_tree = SourceUnit(vec![SourceUnitPart::FunctionDefinition(Box::new(
         FunctionDefinition {
-            doc: vec![],
             loc: Loc::File(0, 17, 32),
             ty: FunctionTy::Function,
             name: Some(Identifier {
@@ -648,7 +477,7 @@ fn test_assembly_parser() {
                                     }],
                                     Some(YulExpression::NumberLiteral(
                                         Loc::File(0, 107, 108),
-                                        BigInt::from(0),
+                                        "0".to_string(), "".to_string(),
                                         None,
                                     )),
                                 ),
@@ -668,7 +497,7 @@ fn test_assembly_parser() {
                                             }],
                                             Some(YulExpression::NumberLiteral(
                                                 Loc::File(0, 148, 149),
-                                                BigInt::from(0),
+                                                "0".to_string(), "".to_string(),
                                                 None,
                                             )),
                                         )],
@@ -848,7 +677,7 @@ fn test_assembly_parser() {
                                         Loc::File(0, 510, 620),
                                         YulExpression::NumberLiteral(
                                             Loc::File(0, 515, 516),
-                                            BigInt::from(0),
+                                            "0".to_string(), "".to_string(),
                                             None,
                                         ),
                                         YulBlock {
@@ -863,12 +692,12 @@ fn test_assembly_parser() {
                                                     arguments: vec![
                                                         YulExpression::NumberLiteral(
                                                             Loc::File(0, 554, 555),
-                                                            BigInt::from(0),
+                                                            "0".to_string(), "".to_string(),
                                                             None,
                                                         ),
                                                         YulExpression::NumberLiteral(
                                                             Loc::File(0, 557, 558),
-                                                            BigInt::from(0),
+                                                            "0".to_string(), "".to_string(),
                                                             None,
                                                         ),
                                                     ],
@@ -972,7 +801,7 @@ fn test_assembly_parser() {
                                                                     arguments: vec![
                                                                         YulExpression::NumberLiteral(
                                                                             Loc::File(0, 924, 930),
-                                                                            BigInt::from(3),
+                                                                            "3".to_string(), "".to_string(),
                                                                             Some(Identifier {
                                                                                 loc: Loc::File(0, 926, 930),
                                                                                 name: "u256".to_string(),
@@ -980,7 +809,7 @@ fn test_assembly_parser() {
                                                                         ),
                                                                         YulExpression::NumberLiteral(
                                                                             Loc::File(0, 932, 938),
-                                                                            BigInt::from(2),
+                                                                            "2".to_string(), "".to_string(),
                                                                             Some(Identifier {
                                                                                 loc: Loc::File(0, 934, 938),
                                                                                 name: "u256".to_string(),
@@ -1044,7 +873,6 @@ fn parse_revert_test() {
 
     let expected_parse_tree = SourceUnit(vec![SourceUnitPart::ContractDefinition(Box::new(
         ContractDefinition {
-            doc: vec![],
             loc: Loc::File(0, 9, 28),
             ty: ContractTy::Contract(Loc::File(0, 9, 17)),
             name: Identifier {
@@ -1054,7 +882,6 @@ fn parse_revert_test() {
             base: vec![],
             parts: vec![
                 ContractPart::ErrorDefinition(Box::new(ErrorDefinition {
-                    doc: vec![],
                     loc: Loc::File(0, 42, 59),
                     name: Identifier {
                         loc: Loc::File(0, 48, 57),
@@ -1063,7 +890,6 @@ fn parse_revert_test() {
                     fields: vec![],
                 })),
                 ContractPart::FunctionDefinition(Box::new(FunctionDefinition {
-                    doc: vec![],
                     loc: Loc::File(0, 73, 89),
                     ty: FunctionTy::Function,
                     name: Some(Identifier {
@@ -1080,10 +906,10 @@ fn parse_revert_test() {
                         unchecked: false,
                         statements: vec![Statement::Revert(
                             Loc::File(0, 107, 125),
-                            Some(Identifier {
+                            Some(Expression::Variable(Identifier {
                                 loc: Loc::File(0, 114, 123),
                                 name: "BAR_ERROR".to_string(),
-                            }),
+                            })),
                             vec![],
                         )],
                     }),
@@ -1125,7 +951,6 @@ fn parse_user_defined_value_type() {
 
     let expected_parse_tree = SourceUnit(vec![
         SourceUnitPart::TypeDefinition(Box::new(TypeDefinition {
-            doc: vec![],
             loc: Loc::File(0, 9, 32),
             name: Identifier {
                 loc: Loc::File(0, 14, 21),
@@ -1134,7 +959,6 @@ fn parse_user_defined_value_type() {
             ty: Expression::Type(Loc::File(0, 25, 32), Type::Uint(256)),
         })),
         SourceUnitPart::ContractDefinition(Box::new(ContractDefinition {
-            doc: vec![],
             loc: Loc::File(0, 42, 61),
             ty: ContractTy::Contract(Loc::File(0, 42, 50)),
             name: Identifier {
@@ -1143,7 +967,6 @@ fn parse_user_defined_value_type() {
             },
             base: vec![],
             parts: vec![ContractPart::TypeDefinition(Box::new(TypeDefinition {
-                doc: vec![],
                 loc: Loc::File(0, 75, 98),
                 name: Identifier {
                     loc: Loc::File(0, 80, 87),
@@ -1156,3 +979,22 @@ fn parse_user_defined_value_type() {
 
     assert_eq!(actual_parse_tree, expected_parse_tree);
 }
+
+#[test]
+fn parse_no_parameters_yul_function() {
+    let src = r#"
+contract C {
+	function testing() pure public {
+		assembly {
+			function multiple() -> ret1, ret2 {
+				ret1 := 1
+				ret2 := 2
+			}
+		}
+	}
+}
+    "#;
+
+    let (actual_parse_tree, _) = crate::parse(src, 0).unwrap();
+    assert_eq!(actual_parse_tree.0.len(), 1);
+}
